/* create_tables.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "create_tables.h"
#include "database.h"

static void
create_artists_table() {
  music_db->sql_command("CREATE TABLE artists ("
			"artist_id INT PRIMARY KEY, "
			"last_name " + music_db->varchar_i(80) + " NOT NULL, "
			"first_name " + music_db->varchar_i(80) + " NOT NULL, "
			"name_suffix TEXT, "
			"group_prefix TEXT, "
			"group_name " + music_db->varchar_i(80) + " NOT NULL)");
  music_db->create_sequence("artists_artist_id_seq");

  music_db->sql_command
    ("CREATE INDEX artists_name ON artists (last_name, first_name, group_name)");
}

static void
create_cds_table() {
  music_db->sql_command
    ("CREATE TABLE cds ("
     "cd_id INT PRIMARY KEY, "
     "cddb CHAR(8), "
     "performer INT, "
     "writer INT, "
     "title TEXT)");
  music_db->create_sequence("cds_cd_id_seq");
}

static void
create_cd_tracks_table() {
  music_db->sql_command
    ("CREATE TABLE cd_tracks ("
     "cd_id INT NOT NULL, "
     "track_num INT NOT NULL, "
     "song_id INT NOT NULL)");
  music_db->sql_command
    ("CREATE INDEX cd_tracks_song_id ON cd_tracks (song_id)");
  music_db->sql_command
    ("CREATE INDEX cd_tracks_track_num ON cd_tracks (cd_id, track_num)");
}

static void
create_songs_table() {
  music_db->sql_command
    ("CREATE TABLE songs ("
     "song_id INT PRIMARY KEY, "
     "performer INT, "
     "writer INT, "
     "length FLOAT, "
     "loaded_time TIMESTAMP, "
     "file_type INT, " 
     "volume_id INT, "
     "name TEXT, "
     "filename TEXT)");
  music_db->create_sequence("songs_song_id_seq");
}

static void
create_songs_convert_table() {
  music_db->sql_command
    ("CREATE TABLE songs_convert ("
     "song_id INT PRIMARY KEY, "
     "new_file_type INT, "
     "new_volume_id INT, "
     "request_time TIMESTAMP, "
     "encode_id INT)");
}

static void
create_songs_convert_failed_table() {
  music_db->sql_command
    ("CREATE TABLE songs_convert_failed ("
     "song_id INT, "
     "new_file_type INT, "
     "new_volume_id INT, "
     "request_time TIMESTAMP, "
     "encode_id INT, "
     "failed_time TIMESTAMP, "
     "failed_reason TEXT)");
}

static void
create_forms_table() {
  music_db->sql_command
    ("CREATE TABLE forms ("
     "form_id INT PRIMARY KEY, "
     "num_rows INT, "
     "num_columns INT, "
     "homogeneous BOOL, "
     "x_size_pixels INT, "
     "y_size_pixels INT, "
     "scrollbar BOOL, "
     "general_sql_command TEXT, "
     "name TEXT)");
  music_db->create_sequence("forms_form_id_seq");
}

static void
create_form_relations_table() {
  music_db->sql_command
    ("CREATE TABLE form_relations ("
     "form_id INT, "
     "table_id INT, "
     "next_form INT)");
}

static void
create_form_queries_table() {
  music_db->sql_command
    ("CREATE TABLE form_queries ("
     "form_id INT PRIMARY KEY, "
     "sequence FLOAT, "
     "menu_item TEXT, "
     "accelerator TEXT)");
}

static void
create_form_widgets_table() {
  music_db->sql_command
    ("CREATE TABLE form_widgets ("
     "widget_id INT PRIMARY KEY, "
     "form_id INT, "
     "table_id INT, "
     "field_id INT, "
     "widget_type INT, "
     "type_assoc INT, "
     "place_left INT, "
     "place_right INT, "
     "place_top INT, "
     "place_bottom INT, "
     "xoptions INT, "
     "yoptions INT, "
     "xpadding INT, "
     "ypadding INT, "
     "uxsize INT, "
     "uysize INT, "
     "internal_name TEXT, "
     "label TEXT)");
  music_db->create_sequence("form_widgets_widget_id_seq");
}

static void
create_combo_boxes_table() {
  music_db->sql_command
    ("CREATE TABLE combo_boxes ("
     "combo_box_id INT PRIMARY KEY, "
     "description TEXT)");
  music_db->create_sequence("combo_boxes_combo_box_id_seq");
}

static void
create_combo_items_table() {
  music_db->sql_command
    ("CREATE TABLE combo_items ("
     "combo_item_id INT PRIMARY KEY, "
     "label TEXT)");
}

static void
create_combo_fields_table() {
  music_db->sql_command
    ("CREATE TABLE combo_fields ("
     "combo_box_id INT, "
     "combo_item_id INT, "
     "sequence INT)");
}

static void
create_ratio_sliders_table() {
  music_db->sql_command
    ("CREATE TABLE ratio_sliders ("
     "widget_id INT, "
     "table_id INT, "
     "field_id INT, "
     "sequence INT)");
}

static void
create_extension_fields_table() {
  music_db->sql_command
    ("CREATE TABLE extension_fields ("
     "field_id INT PRIMARY KEY, "
     "table_id INT, "
     "field_type INT, "
     "foreign_table INT, "
     "default_value TEXT, "
     "field_name TEXT, "
     "label TEXT)");
  music_db->create_sequence("extension_fields_field_id_seq");
}

static void
create_default_forms_table() {
  music_db->sql_command
    ("CREATE TABLE default_forms ("
     "table_id INT PRIMARY KEY, "
     "form_id INT)");
}

static void
create_configure_table() {
  music_db->sql_command
    ("CREATE TABLE configure ("
     "database_version FLOAT, "
     "cdrom_type INT, "
     "cdrom_load_type INT, "
     "cdrom_want_type INT, "
     "cdrom_load_volume INT, "
     "cdrom_want_volume INT)");
  music_db->sql_command
    ("INSERT INTO configure (database_version) VALUES (0.81)");
}

static void
create_volumes_table() {
  music_db->sql_command
    ("CREATE TABLE volumes ("
     "volume_id INT PRIMARY KEY, "
     "bucket_size INT, "
     "current_bucket INT, "
     "description TEXT, "
     "definition TEXT, "
     "bucket_name TEXT)");
  music_db->create_sequence("volumes_volume_id_seq");

  int temp_id = music_db->get_next_sequence("volumes_volume_id_seq");
  music_db->sql_command
    ("INSERT INTO volumes ("
     "volume_id, "
     "bucket_size, current_bucket, "
     "description, definition, bucket_name) VALUES (" +
     DataCell(temp_id) + ", "
     "0, 0, "
     "'Loading area', "
     "'/music_files/wav', "
     "'')");

  int store_id = music_db->get_next_sequence("volumes_volume_id_seq");
  music_db->sql_command
    ("INSERT INTO volumes ("
     "volume_id, "
     "bucket_size, current_bucket, "
     "description, definition, bucket_name) VALUES (" +
     DataCell(store_id) + ", "
     "650, 0, "
     "'MP3 storage', "
     "'/music_files/mp3', "
     "'b$bucket')");

  music_db->sql_command
    ("UPDATE configure SET"
     " cdrom_load_volume=" + DataCell(temp_id) + ", "
     " cdrom_want_volume=" + DataCell(store_id));
}

static void
create_soundfile_exts_table() {
  music_db->sql_command
    ("CREATE TABLE soundfile_exts ("
     "soundfile_id INT, "
     "extension TEXT)");
}

static void
create_soundfile_convert_table() {
  music_db->sql_command
    ("CREATE TABLE soundfile_convert ("
     "from_type INT, "
     "to_type INT, "
     "command TEXT)");
}

static void
create_soundfile_table() {
  music_db->sql_command
    ("CREATE TABLE soundfile ("
     "soundfile_id INT PRIMARY KEY, "
     "sajber_backend BOOL, "
     "description TEXT, "
     "play_command TEXT, "
     "play_response TEXT, "
     "verify_command TEXT, "
     "verify_response TEXT)");
  music_db->create_sequence("soundfile_soundfile_id_seq");

  int mp3_id = music_db->get_next_sequence("soundfile_soundfile_id_seq");
  music_db->sql_command
    ("INSERT INTO soundfile ("
     "soundfile_id, "
     "description, sajber_backend, "
     "play_command, play_response, "
     "verify_command, verify_response) VALUES (" + DataCell(mp3_id) + ", "
     "'MPG3 file', 'f', "
#if 0
     "'bufstream -b 4096 < $filename | mpg123 -v -a $sound_device -k $start_frame -', "
#else
     "'mpg123 -v -a $sound_device -k $start_frame $filename', "
#endif
     "'Frame# $current_frame', "
     "'mpg123 -t -v -n 1 $filename', "
     "'Frame# $current_frame [$frames_remaining]')");

  int wav_id = music_db->get_next_sequence("soundfile_soundfile_id_seq");
  music_db->sql_command
    ("INSERT INTO soundfile ("
     "soundfile_id, "
     "description, sajber_backend, "
     "play_command, play_response, "
     "verify_command, verify_response) VALUES (" + DataCell(wav_id) + ", "
     "'WAV file', 'f', "
#if defined(HAVE_AFOPENNAMEDFD) && 0
     "'bufstream -b 4096 < $filename | vplaysf -v -s $start_time -p $net_time -d $sound_device -', "
#else
     "'vplaysf -v -s $start_time -p $net_time -d $sound_device $filename', "
#endif
     "'time $current_time', "
     "'vplaysf -l $filename', 'is $total_time seconds')");

  int cd_id = music_db->get_next_sequence("soundfile_soundfile_id_seq");
  music_db->sql_command
    ("INSERT INTO soundfile ("
     "soundfile_id, "
     "description, sajber_backend, "
     "play_command, play_response, "
     "verify_command, verify_response) VALUES (" + DataCell(cd_id) + ", "
     "'CD-Rom', 'f', "
     "'vplaycd -v -s $start_time -p $net_time -d $cdrom_device $track', "
     "'time $current_time', "
     "NULL, NULL)");

  music_db->sql_command
    ("INSERT INTO soundfile_exts (soundfile_id, extension) VALUES (" +
     DataCell(mp3_id) + ", '.mp3')");
  music_db->sql_command
    ("INSERT INTO soundfile_exts (soundfile_id, extension) VALUES (" +
     DataCell(wav_id) + ", '.wav')");

  music_db->sql_command
    ("INSERT INTO soundfile_convert (from_type, to_type, command) VALUES (" +
     DataCell(mp3_id) + ", " + DataCell(wav_id) + ", "
     "'mpg123 -w $to $from')");
  music_db->sql_command
    ("INSERT INTO soundfile_convert (from_type, to_type, command) VALUES (" +
     DataCell(cd_id) + ", " + DataCell(wav_id) + ", "
     "'cdparanoia -d $cdrom_device $track $to')");
  music_db->sql_command
    ("INSERT INTO soundfile_convert (from_type, to_type, command) VALUES (" +
     DataCell(wav_id) + ", " + DataCell(mp3_id) + ", "
     "'8hz-mp3 $from $to')");

  music_db->sql_command
    ("UPDATE configure SET cdrom_type=" + DataCell(cd_id));
  music_db->sql_command
    ("UPDATE configure SET cdrom_load_type=" + DataCell(wav_id));
  music_db->sql_command
    ("UPDATE configure SET cdrom_want_type=" + DataCell(mp3_id));
}

void
create_tables() {
  if (!music_db->has_table("artists")) {
    create_artists_table();
  }
  if (!music_db->has_table("cds")) {
    create_cds_table();
  }
  if (!music_db->has_table("cd_tracks")) {
    create_cd_tracks_table();
  }
  if (!music_db->has_table("songs")) {
    create_songs_table();
  } 
  if (!music_db->has_table("songs_convert")) {
    create_songs_convert_table();
  } 
  if (!music_db->has_table("songs_convert_failed")) {
    create_songs_convert_failed_table();
  } 
  if (!music_db->has_table("forms")) {
    create_forms_table();
  } 
  if (!music_db->has_table("form_relations")) {
    create_form_relations_table();
  } 
  if (!music_db->has_table("form_queries")) {
    create_form_queries_table();
  } 
  if (!music_db->has_table("form_widgets")) {
    create_form_widgets_table();
  } 
  if (!music_db->has_table("combo_boxes")) {
    create_combo_boxes_table();
  } 
  if (!music_db->has_table("combo_items")) {
    create_combo_items_table();
  } 
  if (!music_db->has_table("combo_fields")) {
    create_combo_fields_table();
  } 
  if (!music_db->has_table("ratio_sliders")) {
    create_ratio_sliders_table();
  } 
  if (!music_db->has_table("extension_fields")) {
    create_extension_fields_table();
  }
  if (!music_db->has_table("default_forms")) {
    create_default_forms_table();
  }
  if (!music_db->has_table("configure")) {
    create_configure_table();
  }
  if (!music_db->has_table("volumes")) {
    create_volumes_table();
  }
  if (!music_db->has_table("soundfile_exts")) {
    create_soundfile_exts_table();
  }
  if (!music_db->has_table("soundfile_convert")) {
    create_soundfile_convert_table();
  }
  if (!music_db->has_table("soundfile")) {
    create_soundfile_table();
  }
}
