/* mySQLQueryResult.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef MYSQLQUERYRESULT_H
#define MYSQLQUERYRESULT_H

#ifdef USE_MYSQL

#include "queryResult.h"

#include <mysql.h>

class MySQLDatabase;

class MySQLQueryResult : public QueryResultBase {
public:
  MySQLQueryResult(MySQLDatabase &db);
  virtual ~MySQLQueryResult();

  virtual bool success() const;
  virtual string get_error_message() const;

  virtual int get_num_columns() const;
  virtual string get_column_name(int c) const;
  virtual DataCell::Type get_column_type(int c) const;
  
  virtual int get_num_rows() const;
  virtual DataCell get_cell(int r, int c) const;
  const char *get_cell_raw(int r, int c) const;

  void set_my_result(MYSQL_RES *result);
  void set_error_message(const string &msg);
  
private:
  MySQLDatabase &get_db() const { return (MySQLDatabase &)_db; }
  void set_row(int r);

  bool _has_error;
  MYSQL_RES *_my_result;
  MYSQL_FIELD *_columns;
  int _num_columns;

  int _current_row_index;
  MYSQL_ROW _current_row;

  string _error_message;
};

#endif // USE_MYSQL

#endif
